@@ -314,6 +314,11 @@ ImportScanFinding = collections.namedtuple(
     "ImportScanFinding", ("found_in", "priority", "full_path", "search_order")
 )
 
+# We put here things that are not worth it (Cython is not really used by
+# anything really, or where it's know to not have a big # impact, e.g. lxml.
+
+unworthy_namespaces = ("Cython", "lxml")
+
 
 def _reportCandidates(module_name, candidate, candidates):
     if (
@@ -326,12 +331,14 @@ def _reportCandidates(module_name, candidate, candidates):
                 continue
 
             if c.search_order == candidate.search_order:
-                recursion_logger.info(
-                    """\
+                if not module_name.hasOneOfNamespaces(unworthy_namespaces):
+
+                    recursion_logger.info(
+                        """\
 Should decide '--prefer-source-code' vs. '--no-prefer-source-code', using \
 existing '%s' extension module by default. Candidates were: %s <-> %s."""
-                    % (module_name, candidate, c)
-                )
+                        % (module_name, candidate, c)
+                    )
 
 
 def _findModuleInPath2(module_name, search_path):
